﻿using System;
using System.Collections;
using Assets.Scripts.Values;
using UnityEngine;

public class PlayerContoller : MonoBehaviour
{
    [Header("Objects")]
    public GameObject Stere;
    public ParticleSystem HitParticle;
    public UnityEngine.UI.Image DamgeLvl;

    [Header("Player Settings")]
    public float Speed;
    public float StereLimit;
    public float StereBackSpeed;
    public float ShipsMaxFuel = 100f;
    public float ShipMaxLife;
    public float ShipTurnAngle = 15.0f;

    public bool IsShipDestroyed { get; private set; }
    public bool HasFuel => CurrentFuel > 0;

    public float CurrentFuel {
        get => _currentFuel;
        set => _currentFuel = Mathf.Min(ShipsMaxFuel, value);
    }

    private Rigidbody rigid;
    private float inputValue;
    private GameManager _gameManager;
    private float _currentFuel;
    private ParticleSystem particleTrial;
    private float currentLife;
    private ShipSpeedType currentSpeed;

    void Awake()
    {
        rigid = GetComponent<Rigidbody>();
        particleTrial = GetComponentInChildren<ParticleSystem>();
        Stere.transform.rotation = Quaternion.identity;
        _gameManager = GameObject.FindGameObjectWithTag(Tags.GameManager)
            .GetComponent<GameManager>();

        IsShipDestroyed = false;
    }

    void Start()
    {
        CurrentFuel = _gameManager.StartShipFuel;
        currentLife = ShipMaxLife;
        DamgeLvl.color = Color.green;
        Stere.transform.rotation = Quaternion.Euler(0f, 0f, 0f);
        StartCoroutine(LoseFuel());
    }

    void Update()
    {

        if (Input.GetButton("Cancel"))
        {
            GameOver(() => _gameManager.GraczSiePoddal());
        }

#if UNITY_IPHONE || UNITY_ANDROID
        Vector3 dir = Vector3.zero;
        dir.x = Input.acceleration.x * 5.0f;
       
        if (dir.sqrMagnitude > 1)
            dir.Normalize();

        inputValue = dir.x;
#else 
        inputValue = Input.GetAxis("Horizontal");
#endif
    }

    void FixedUpdate()
    {
        FreezeRotationXZ();

        if(HasFuel)
        {
            ControlleShip();
        }

        ControlleStere();

        if (IsShipDestroyed)
        {
            transform.Rotate(Vector3.back, 90);
        }
    }

    public void Harm(HarmTypes harm)
    {
        if (harm == HarmTypes.IslandCollision)
        {
            if (currentSpeed == ShipSpeedType.Mid)
            {
                //HitParticle.Play();
                currentLife -= HarmHelper.GetHarmValue(harm);
            }
            else if (currentSpeed == ShipSpeedType.Hig)
            {
                //HitParticle.Play();
                currentLife = 0;
            }
        }

        if (currentLife >= 70f)
        {
            DamgeLvl.color = Color.green;
        }
        else if (currentLife < 70f && currentLife >= 40f)
        {
            DamgeLvl.color = Color.yellow;
        }
        else
        {
            DamgeLvl.color = Color.red;
        }

        if (currentLife <= 0)
        {
            GameOver(() => _gameManager.ZapierdolilOWyspe(gameObject));
        }
    }

    public void SetSpeedIndicator(ShipSpeedType speedType)
    {
        var emission = particleTrial.emission;
        var main = particleTrial.main;
        switch (speedType)
        {
            case ShipSpeedType.Low:
                emission.rateOverTimeMultiplier = 5f;
                main.startSpeedMultiplier = 0.5f;
                break;
            case ShipSpeedType.Mid:
                emission.rateOverTimeMultiplier = 50f;
                main.startSpeedMultiplier = 1.5f;
                break;
            case ShipSpeedType.Hig:
                emission.rateOverTimeMultiplier = 200f;
                main.startSpeedMultiplier = 3f;
                break;
        }

        currentSpeed = speedType;
    }

    public void GameOver(Action casue)
    {
        IsShipDestroyed = true;
        StartCoroutine(SimulateDeath(casue));
    }

    private IEnumerator SimulateDeath(Action casue)
    {
        yield return new WaitForSeconds(1f);
        rigid.constraints = RigidbodyConstraints.None;

        yield return new WaitForSeconds(1f);

        casue();
    }

    private void ControlleShip()
    {
        if (IsShipDestroyed)
            return;

        float forwardForceAmplifire = Speed - Math.Abs(inputValue);

        // When player turn, reduce forward force to half.
        if (inputValue != 0)
        {
            rigid.AddForce((transform.forward * -1) * forwardForceAmplifire / 2, ForceMode.Force);
        }

        rigid.AddForce(transform.forward * forwardForceAmplifire, ForceMode.Force);
        rigid.AddTorque(Vector3.up * (inputValue / 2));

        if (inputValue != 0) {

            Quaternion rot = Quaternion.Euler(0, transform.rotation.eulerAngles.y, inputValue * -ShipTurnAngle);
            transform.rotation = rot;
        }
    }

    private IEnumerator LoseFuel()
    {
        while (!IsShipDestroyed && HasFuel)
        {
            yield return new WaitForSeconds(1f);
            CurrentFuel -= _gameManager.FuelLossRatePerSecond;

            if(CurrentFuel <= 0)
            {
                FuelHasEnded();
            }
        }
    }

    private void FuelHasEnded()
    {
        Speed = 0f;
        GameOver(() => _gameManager.EndOfFuel());
    }

    private void ControlleStere()
    {
        if (IsShipDestroyed)
            return;

        if (NeedBackToZero())
        {
            Stere.transform.rotation = Quaternion.RotateTowards(
                Stere.transform.rotation,
                transform.rotation,
                StereBackSpeed * Time.deltaTime);
        }

        float eulerY = Stere.transform.localRotation.y;

        if (IsStereLimit(eulerY))
        {
            return;
        }

        Vector3 sterePosition = Stere.transform.position;
        Stere.transform.RotateAround(sterePosition, Vector3.up, inputValue);
    }

    private void FreezeRotationXZ()
    {
        // Block rotation on X and Z.
        Quaternion rot = Quaternion.Euler(0, transform.rotation.eulerAngles.y, 0);
        transform.rotation = rot;
    }

    private bool IsStereLimit(float eulerY)
    {
        if (eulerY > StereLimit && inputValue > 0)
        {
            return true;
        }

        if (eulerY < -StereLimit && inputValue < 0)
        {
            return true;
        }

        return false;
    }

    private bool NeedBackToZero()
    {
        return Math.Abs(inputValue) < 0.1f;
    }
}
