﻿using Assets.Scripts.Values;
using UnityEngine;
using UnityEngine.UI;

namespace Assets.Scripts.UI
{
    public class Pedometer: MonoBehaviour
    {
        public Image HealthImage;
        private GameManager _controller;
        private PlayerContoller _player;
        private float _maxHeight;
        private Vector2 _anchoredPositionStart;

        private void Awake()
        {
            _controller = GameObject.FindGameObjectWithTag(Tags.GameManager)
                .GetComponent<GameManager>();

            _player = GameObject.FindGameObjectWithTag(Tags.Player)
                .GetComponent<PlayerContoller>();
        }

        private void Start()
        {
            _maxHeight = HealthImage.rectTransform.sizeDelta.y;
            _anchoredPositionStart = new Vector2(HealthImage.rectTransform.anchoredPosition.x, 
                HealthImage.rectTransform.anchoredPosition.y - HealthImage.rectTransform.sizeDelta.y/2);
            HealthImage.rectTransform.anchoredPosition = _anchoredPositionStart;
        }

        private void Update()
        {
            float magnitude = _player.GetComponent<Rigidbody>().velocity.magnitude;
            Color colorToSet;

            if(magnitude > _controller.DeathSpeed)
            {
                colorToSet = Color.red;
                _player.SetSpeedIndicator(ShipSpeedType.Hig);
            }
            else if(magnitude < _controller.DeathSpeed && magnitude > _controller.DangerousSpeed)
            {
                colorToSet = Color.Lerp(Color.yellow, Color.red, (magnitude - _controller.DangerousSpeed) / (_controller.DeathSpeed - _controller.DangerousSpeed));
                _player.SetSpeedIndicator(ShipSpeedType.Mid);
            }
            else
            {
                colorToSet = Color.Lerp(Color.green, Color.yellow, magnitude / _controller.DangerousSpeed);
                _player.SetSpeedIndicator(ShipSpeedType.Low);
            }


            HealthImage.color = colorToSet;
            float heightBasedOnSpeed = (magnitude / _controller.DeathSpeed) * _maxHeight;
            HealthImage.rectTransform.sizeDelta = new Vector2(HealthImage.rectTransform.sizeDelta.x, Mathf.Min(heightBasedOnSpeed, _maxHeight));

            float finalPosition = _anchoredPositionStart.y + HealthImage.rectTransform.sizeDelta.y/2;
            HealthImage.rectTransform.anchoredPosition = new Vector2(HealthImage.rectTransform.anchoredPosition.x, finalPosition);
        }
    }
}
