﻿using System.Collections;
using UnityEngine;

namespace Assets.Scripts
{
    public class WindController: MonoBehaviour
    {
        public float MinimumForceTime = 5;
        public float MaximumForceTime = 10;
        public float ForceModyfierMinimum = 3;
        public float ForceModyfierMaximum = 6;
        public float ForceChangeTendencyMinimum = 10f;
        public float ForceChangeTendencyMaximum = 40f;
        public int WindSwitchMinimumValue = 3;
        public int WindSwitchMaximumValue = 7;

        public Vector3 ForceVector;
        public GameObject ToAddForce;

        private int _windSwitches = 0;
        private int _nextWindSwitchIn = 0;
        private Vector3 _forceDirection;

        private void Awake()
        {
            ForceVector = _forceDirection = transform.forward;
            StartCoroutine(WindLoop());
        }

        private void Update()
        {
            ToAddForce.GetComponent<Rigidbody>().AddForce(ForceVector, ForceMode.Force);
            //Debug.Log("X:" + ForceVector.x + "- Z:" + ForceVector.z);
        }

        private float ChangeForceParameters()
        {
            float nextForce = Random.Range(ForceModyfierMinimum, ForceModyfierMaximum);
            float nextTimeChange = Random.Range(MinimumForceTime, MaximumForceTime);
            float nextWindChange;

            if (_windSwitches == _nextWindSwitchIn)
            {
                _windSwitches = 0;
                _nextWindSwitchIn = Random.Range(WindSwitchMinimumValue, WindSwitchMaximumValue);
                nextWindChange = -Random.Range(120, 170);
            }
            else
            {
                _windSwitches++;
                nextWindChange = Random.Range(ForceChangeTendencyMinimum, ForceChangeTendencyMaximum);
            }


            _forceDirection = Quaternion.AngleAxis(nextWindChange, Vector3.up) * _forceDirection;
            ForceVector = _forceDirection * nextForce;

            return nextTimeChange;
        }

        private IEnumerator WindLoop()
        {
            while (true)
            {
                float nextWindChange = ChangeForceParameters();
                yield return new WaitForSeconds(nextWindChange);
            }
        }
    }
}
